import logging
import os
from logging.config import fileConfig

from alembic import context
from core.database import SQLALCHEMY_DATABASE_URL, Base
from core.settings import config as ssa_config
from environment.checks import PlatformEnvironment
from sqlalchemy import engine_from_config, pool

LOG_LEVEL = ssa_config.LOGGING.LEVEL
LOG_BACKUP_COUNT = ssa_config.LOGGING.ROTATION_LIMIT
LOG_MAX_BYTES = ssa_config.LOGGING.MAX_FILE_SIZE

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config  # pylint: disable=no-member

if PlatformEnvironment.is_linux():
    LOG_FILE_PATH = "/var/log/termidesk/session-agent.log"

if PlatformEnvironment.is_windows():
    PROGRAM_DATA = os.path.expandvars(r"%PROGRAMDATA%")
    LOG_FILE_PATH = os.path.join(
        PROGRAM_DATA, "UVEON", "Termidesk Session Agent", "log", "session-agent.log"
    )

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name is not None:
    fileConfig(config.config_file_name)
else:
    # NOTE: на этапе миграций логгер сессионного агента не сконфигурирован.
    log_conf = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "console": {
                "format": "\x1b[33;20m%(levelname)-5.5s [%(name)s] %(message)s\x1b[0m",
                "datefmt": "%H:%M:%S",
            },
            "file": {
                "format": "%(asctime)s.%(msecs)03d %(levelname)-6s %(name)s - %(message)s",
            },
        },
        "handlers": {
            "console": {
                "formatter": "console",
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stderr",
                "level": "NOTSET",
            },
            "file": {
                "formatter": "file",
                "class": "logging_config.CompressingFileHandler",
                "filename": LOG_FILE_PATH,
                "maxBytes": LOG_MAX_BYTES,
                "backupCount": LOG_BACKUP_COUNT,
            },
        },
        "loggers": {
            "alembic": {
                "handlers": [],
                "level": 20,
                "propagate": True,
            },
        },
        "root": {
            "handlers": ["console", "file"],
            "level": LOG_LEVEL,
            "propagate": False,
        },
    }

    logging.config.dictConfig(log_conf)
# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def get_url_sqlite():
    return SQLALCHEMY_DATABASE_URL


def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = get_url_sqlite()
    context.configure(  # pylint: disable=no-member
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():  # pylint: disable=no-member
        context.run_migrations()  # pylint: disable=no-member


def run_migrations_online() -> None:
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    configuration = config.get_section(config.config_ini_section, {})
    configuration["sqlalchemy.url"] = get_url_sqlite()
    connectable = engine_from_config(
        configuration,
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(  # pylint: disable=no-member
            connection=connection, target_metadata=target_metadata
        )

        with context.begin_transaction():  # pylint: disable=no-member
            context.run_migrations()  # pylint: disable=no-member


if context.is_offline_mode():  # pylint: disable=no-member
    run_migrations_offline()
else:
    run_migrations_online()
